@@ -184,6 +184,8 @@ struct supplicant_task {
 	enum supplicant_state state;
 	gboolean scanning;
 	GSList *scan_results;
+	DBusPendingCall *scan_call;
+	DBusPendingCall *result_call;
 	struct iw_range *range;
 	gboolean disconnecting;
 };
@@ -910,6 +912,8 @@ static void scan_reply(DBusPendingCall *call, void *user_data)
 
 	DBG("task %p", task);
 
+	task->scan_call = NULL;
+
 	reply = dbus_pending_call_steal_reply(call);
 	if (reply == NULL)
 		return;
@@ -930,10 +934,12 @@ done:
 static int initiate_scan(struct supplicant_task *task)
 {
 	DBusMessage *message;
-	DBusPendingCall *call;
 
 	DBG("task %p", task);
 
+	if (task->scan_call != NULL)
+		return -EALREADY;
+
 	message = dbus_message_new_method_call(SUPPLICANT_NAME, task->path,
 					SUPPLICANT_INTF ".Interface", "scan");
 	if (message == NULL)
@@ -942,19 +948,19 @@ static int initiate_scan(struct supplicant_task *task)
 	dbus_message_set_auto_start(message, FALSE);
 
 	if (dbus_connection_send_with_reply(connection, message,
-						&call, TIMEOUT) == FALSE) {
+					&task->scan_call, TIMEOUT) == FALSE) {
 		connman_error("Failed to initiate scan");
 		dbus_message_unref(message);
 		return -EIO;
 	}
 
-	if (call == NULL) {
+	if (task->scan_call == NULL) {
 		connman_error("D-Bus connection not available");
 		dbus_message_unref(message);
 		return -EIO;
 	}
 
-	dbus_pending_call_set_notify(call, scan_reply, task, NULL);
+	dbus_pending_call_set_notify(task->scan_call, scan_reply, task, NULL);
 
 	dbus_message_unref(message);
 
@@ -1193,7 +1199,6 @@ static void properties_reply(DBusPendingCall *call, void *user_data)
 	unsigned short channel, frequency;
 	const char *mode, *security;
 	char *group = NULL;
-	unsigned int ssid_len;
 
 	DBG("task %p", task);
 
@@ -1331,10 +1336,8 @@ static void properties_reply(DBusPendingCall *call, void *user_data)
 	if (result.name != NULL && result.name[0] != '\0')
 		connman_network_set_name(network, result.name);
 
-	if (connman_network_get_blob(network, "WiFi.SSID", &ssid_len) == NULL) {
-		connman_network_set_blob(network, "WiFi.SSID",
-					 result.ssid, result.ssid_len);
-	}
+	connman_network_set_blob(network, "WiFi.SSID",
+						result.ssid, result.ssid_len);
 
 	connman_network_set_string(network, "WiFi.Mode", mode);
 
@@ -1368,7 +1371,6 @@ done:
 static void get_properties(struct supplicant_task *task)
 {
 	DBusMessage *message;
-	DBusPendingCall *call;
 	char *path;
 
 	path = g_slist_nth_data(task->scan_results, 0);
@@ -1388,25 +1390,28 @@ static void get_properties(struct supplicant_task *task)
 	dbus_message_set_auto_start(message, FALSE);
 
 	if (dbus_connection_send_with_reply(connection, message,
-						&call, TIMEOUT) == FALSE) {
+				&task->result_call, TIMEOUT) == FALSE) {
 		connman_error("Failed to get network properties");
 		dbus_message_unref(message);
 		goto noscan;
 	}
 
-	if (call == NULL) {
+	if (task->result_call == NULL) {
 		connman_error("D-Bus connection not available");
 		dbus_message_unref(message);
 		goto noscan;
 	}
 
-	dbus_pending_call_set_notify(call, properties_reply, task, NULL);
+	dbus_pending_call_set_notify(task->result_call,
+					properties_reply, task, NULL);
 
 	dbus_message_unref(message);
 
 	return;
 
 noscan:
+	task->result_call = NULL;
+
 	if (task->scanning == TRUE) {
 		connman_device_set_scanning(task->device, FALSE);
 		task->scanning = FALSE;
@@ -1467,6 +1472,8 @@ done:
 	dbus_message_unref(reply);
 
 noscan:
+	task->result_call = NULL;
+
 	if (task->scanning == TRUE) {
 		connman_device_set_scanning(task->device, FALSE);
 		task->scanning = FALSE;
@@ -1476,10 +1483,12 @@ noscan:
 static void scan_results_available(struct supplicant_task *task)
 {
 	DBusMessage *message;
-	DBusPendingCall *call;
 
 	DBG("task %p", task);
 
+	if (task->result_call != NULL)
+		return;
+
 	message = dbus_message_new_method_call(SUPPLICANT_NAME, task->path,
 						SUPPLICANT_INTF ".Interface",
 							"scanResults");
@@ -1489,12 +1498,12 @@ static void scan_results_available(struct supplicant_task *task)
 	dbus_message_set_auto_start(message, FALSE);
 
 	if (dbus_connection_send_with_reply(connection, message,
-						&call, TIMEOUT) == FALSE) {
+				&task->result_call, TIMEOUT) == FALSE) {
 		connman_error("Failed to request scan result");
 		goto done;
 	}
 
-	if (call == NULL) {
+	if (task->result_call == NULL) {
 		connman_error("D-Bus connection not available");
 		goto done;
 	}
@@ -1502,7 +1511,8 @@ static void scan_results_available(struct supplicant_task *task)
 	if (task->scanning == TRUE)
 		connman_device_set_scanning(task->device, TRUE);
 
-	dbus_pending_call_set_notify(call, scan_results_reply, task, NULL);
+	dbus_pending_call_set_notify(task->result_call,
+					scan_results_reply, task, NULL);
 
 done:
 	dbus_message_unref(message);
@@ -1566,6 +1576,27 @@ static int task_connect(struct supplicant_task *task)
 	return -EINPROGRESS;
 }
 
+static void scanning(struct supplicant_task *task, DBusMessage *msg)
+{
+	DBusError error;
+	dbus_bool_t scanning;
+
+	dbus_error_init(&error);
+
+	if (dbus_message_get_args(msg, &error, DBUS_TYPE_BOOLEAN, &scanning,
+						DBUS_TYPE_INVALID) == FALSE) {
+		if (dbus_error_is_set(&error) == TRUE) {
+			connman_error("%s", error.message);
+			dbus_error_free(&error);
+		} else
+			connman_error("Wrong arguments for scanning");
+		return;
+	}
+
+	connman_info("%s scanning %s", task->ifname,
+				scanning == TRUE ? "started" : "finished");
+}
+
 static void state_change(struct supplicant_task *task, DBusMessage *msg)
 {
 	DBusError error;
@@ -1589,6 +1620,8 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 
 	DBG("state %s ==> %s", oldstate, newstate);
 
+	connman_info("%s %s", task->ifname, newstate);
+
 	state = string2state(newstate);
 	if (state == WPA_INVALID)
 		return;
@@ -1627,7 +1660,8 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 				task->network = task->pending_network;
 				task->pending_network = NULL;
 				task_connect(task);
-			}
+			} else
+				task->network = NULL;
 		}
 		break;
 
@@ -1646,7 +1680,8 @@ static void state_change(struct supplicant_task *task, DBusMessage *msg)
 				task->network = task->pending_network;
 				task->pending_network = NULL;
 				task_connect(task);
-			}
+			} else
+				task->network = NULL;
 		}
 		break;
 
@@ -1682,6 +1717,8 @@ static DBusHandlerResult supplicant_filter(DBusConnection *conn,
 
 	if (g_str_equal(member, "ScanResultsAvailable") == TRUE)
 		scan_results_available(task);
+	else if (g_str_equal(member, "Scanning") == TRUE)
+		scanning(task, msg);
 	else if (g_str_equal(member, "StateChange") == TRUE)
 		state_change(task, msg);
 
@@ -1752,6 +1789,19 @@ int supplicant_stop(struct connman_device *device)
 
 	task_list = g_slist_remove(task_list, task);
 
+	if (task->scan_call != NULL) {
+		dbus_pending_call_cancel(task->scan_call);
+		task->scan_call = NULL;
+	}
+
+	if (task->result_call != NULL) {
+		dbus_pending_call_cancel(task->result_call);
+		task->result_call = NULL;
+	}
+
+	if (task->scanning == TRUE)
+		connman_device_set_scanning(task->device, FALSE);
+
 	disable_network(task);
 
 	remove_network(task);
@@ -1960,10 +2010,7 @@ int supplicant_register(struct supplicant_driver *driver)
 
 	driver_list = g_slist_append(driver_list, driver);
 
-	if (g_dbus_check_service(connection, SUPPLICANT_NAME) == TRUE)
-		supplicant_probe(connection, NULL);
-	else
-		supplicant_activate(connection);
+	supplicant_activate(connection);
 
 	return 0;
 }
